@@ -20,31 +20,34 @@ header files or Java imports.
 Please find the README.rst for more information.
 """
 from __future__ import print_function
-
-VERSION = "1.8.1"
+import sys
+if sys.version[0] == '2':
+    from future_builtins import map, filter  # pylint: disable=W0622, F0401
 
 import itertools
 import re
-import sys
 import os
-import fnmatch
+from fnmatch import fnmatch
 import hashlib
 
+VERSION = "1.8.2"
 
 DEFAULT_CCN_THRESHOLD = 15
 
 
-def analyze(paths, exclude_pattern=[], threads=1, extensions=[]):
+def analyze(paths, exclude_pattern=None, threads=1, extensions=None):
     '''
     returns an iterator of file infomation that contains function
     statistics.
     '''
-    files = get_all_source_files(exclude_pattern, paths)
+    exclude_pattern = exclude_pattern or []
+    extensions = extensions or []
+    files = get_all_source_files(paths, exclude_pattern)
     file_analyzer = FileAnalyzer(extensions)
     return map_files_to_analyzer(files, file_analyzer, threads)
 
 
-def createCommandLineParser(prog=None):
+def create_command_line_parser(prog=None):
     from argparse import ArgumentParser
     parser = ArgumentParser(prog=prog)
     parser.add_argument('paths', nargs='*', default=['.'],
@@ -129,11 +132,11 @@ def createCommandLineParser(prog=None):
     return parser
 
 
-class FunctionInfo(object):
+class FunctionInfo(object):  # pylint: disable=R0902
 
     def __init__(self, name, filename, start_line=0, ccn=1):
         self.cyclomatic_complexity = ccn
-        self.nloc = 0
+        self.nloc = 1
         self.token_count = 1  # the first token
         self.name = name
         self.long_name = name
@@ -169,182 +172,127 @@ class FunctionInfo(object):
             " params (%(nloc)d NLOC, %(token_count)d tokens)") % self.__dict__
 
 
-class FileInformation(object):
+class FileInformation(object):  # pylint: disable=R0903
 
-    def __init__(self, filename, nloc, function_list):
+    def __init__(self, filename, nloc, function_list=None):
         self.filename = filename
         self.nloc = nloc
-        self.function_list = function_list
+        self.function_list = function_list or []
         self.token_count = 0
 
-    average_NLOC = property(lambda self: self._functions_average("nloc"))
+    average_NLOC = property(lambda self: self.functions_average("nloc"))
     average_token = property(
-        lambda self: self._functions_average("token_count"))
+        lambda self: self.functions_average("token_count"))
     average_CCN = property(
-        lambda self: self._functions_average("cyclomatic_complexity"))
+        lambda self: self.functions_average("cyclomatic_complexity"))
     CCN = property(
         lambda self:
         sum(fun.cyclomatic_complexity for fun in self.function_list))
 
-    def _functions_average(self, att):
+    def functions_average(self, att):
         return (sum(getattr(fun, att) for fun in self.function_list)
                 / len(self.function_list) if self.function_list else 0)
 
 
 class CodeInfoContext(object):
-    """
-        provides the builder to build FileInformation
-        The building blocks are:
-        START_NEW_FUNCTION
-            ADD_TO_FUNCTION_NAME
-            ADD_TO_LONG_FUNCTION_NAME
-                PARAMETER
-                    CONDITION
-                    TOKEN
-        END_OF_FUNCTION
-    """
 
     def __init__(self, filename):
-        self.fileinfo = FileInformation(filename, 0, [])
+        self.fileinfo = FileInformation(filename, 0)
         self.current_line = 0
-        self.START_NEW_FUNCTION('')
+        self.current_function = FunctionInfo('', '', 0)
         self.forgive = False
-        self.reader = (CodeReader.get_reader(filename) or CLikeReader)()
-
-    def START_NEW_FUNCTION(self, name):
         self.newline = True
+
+    def add_nloc(self, count):
+        self.current_function.nloc += count
+        self.fileinfo.nloc += count
+
+    def start_new_function(self, name):
         self.current_function =\
             FunctionInfo(name, self.fileinfo.filename, self.current_line)
 
-    def CONDITION(self, inc=1):
+    def add_condition(self, inc=1):
         self.current_function.cyclomatic_complexity += inc
 
-    def ADD_TO_LONG_FUNCTION_NAME(self, app):
+    def add_to_long_function_name(self, app):
         self.current_function.add_to_long_name(app)
 
-    def ADD_TO_FUNCTION_NAME(self, app):
+    def add_to_function_name(self, app):
         self.current_function.add_to_function_name(app)
 
-    def PARAMETER(self, token):
+    def parameter(self, token):
         self.current_function.add_parameter(token)
 
-    def END_OF_FUNCTION(self):
+    def end_of_function(self):
         if not self.forgive:
             self.fileinfo.function_list.append(self.current_function)
         self.forgive = False
-        self.START_NEW_FUNCTION('')
+        self.current_function = FunctionInfo('', '', 0)
 
 
-class Preprocessor(object):
+def preprocessing(tokens, reader):
+    if hasattr(reader, "preprocess"):
+        return reader.preprocess(tokens)
+    else:
+        return (t for t in tokens if not t.isspace() or t == '\n')
 
-    def extend_tokens(self, tokens, context):
-        if hasattr(context.reader, "preprocess"):
-            return context.reader.preprocess(tokens, context)
+
+def comment_counter(tokens, reader):
+    get_comment = reader.get_comment_from_token
+    for token in tokens:
+        comment = get_comment(token)
+        if comment is not None:
+            for _ in comment.splitlines()[1:]:
+                yield '\n'
+            if comment.strip().startswith("#lizard forgive"):
+                reader.context.forgive = True
         else:
-            return (t for t in tokens if not t.isspace() or t == '\n')
-
-
-class CommentCounter(object):
-
-    def extend_tokens(self, tokens, context):
-        get_comment = context.reader.get_comment_from_token
-        for token in tokens:
-            comment = get_comment(token)
-            if comment is not None:
-                if len(comment.splitlines()) > 1:
-                    yield '\n'
-                if comment.strip().startswith("#lizard forgive"):
-                    context.forgive = True
-            else:
-                yield token
-
-
-class LineCounter(object):
-    '''
-    And extension of lizard, to count the line and nloc.
-    '''
-    FUNCTION_CAPTION = "  NLOC  "
-    FUNCTION_INFO_PART = "nloc"
-
-    def extend_tokens(self, tokens, context):
-        context.current_line = 1
-        for token in tokens:
-            context.current_line += (
-                1 if token == '\n' else (len(token.splitlines()) - 1))
-            if token == "\n":
-                self._new_line(context)
-            else:
-                yield token
-
-    def _new_line(self, context):
-        context.fileinfo.nloc += 1
-        context.newline = True
-
-
-class TokenCounter(object):
-    '''
-    An extension that counts the token.
-    But because of historical and implementation reason, the token
-    counting is already done in LineCounter. So this class only do
-    the output.
-    '''
-    FUNCTION_CAPTION = " token "
-    FUNCTION_INFO_PART = "token_count"
-
-    def extend_tokens(self, tokens, context):
-        for token in tokens:
-            self._count_token(context)
-            yield token
-
-    def _count_token(self, context):
-        context.fileinfo.token_count += 1
-        if context.newline:
-            context.current_function.nloc += 1
-            context.newline = False
-            context.current_function.end_line = context.current_line
-        context.current_function.token_count += 1
-
-
-class ParameterCounter(object):
-    ''' An extension that counts the parameters.
-        But because of historical and implementation reason, the parameter
-        counting is already done in FunctionCounter. So this class only do
-        the output.
-    '''
-
-    FUNCTION_CAPTION = " PARAM "
-    FUNCTION_INFO_PART = "parameter_count"
-
-    def extend_tokens(self, tokens, context):
-        return tokens
-
-
-class ConditionCounter(object):
-
-    FUNCTION_CAPTION = "  CCN  "
-    FUNCTION_INFO_PART = "cyclomatic_complexity"
-    conditions = set(['if', 'for', 'while', '&&', '||', '?', 'catch',
-                     'case', '#if', '#ifdef', '#elif'])
-
-    def extend_tokens(self, tokens, context):
-        conditions = (
-            context.reader.conditions
-            if hasattr(context.reader, "conditions") else self.conditions)
-        for token in tokens:
-            if token in conditions:
-                context.CONDITION()
             yield token
 
 
-class SwitchCasesAsOneConditionCounter(object):
-
-    def extend_tokens(self, tokens, context):
-        for token in tokens:
-            if token == 'switch':
-                context.CONDITION()
-            elif token == 'case':
-                context.CONDITION(-1)
+def line_counter(tokens, reader):
+    reader.context.current_line = 1
+    for token in tokens:
+        if token != "\n":
+            count = token.count('\n')
+            reader.context.current_line += count
+            reader.context.add_nloc(count)
             yield token
+        else:
+            reader.context.current_line += 1
+            reader.context.newline = True
+
+
+def token_counter(tokens, reader):
+    for token in tokens:
+        reader.context.fileinfo.token_count += 1
+        if reader.context.newline:
+            reader.context.add_nloc(1)
+            reader.context.newline = False
+        reader.context.current_function.end_line = reader.context.current_line
+        reader.context.current_function.token_count += 1
+        yield token
+
+
+def condition_counter(tokens, reader):
+    if hasattr(reader, "conditions"):
+        conditions = reader.conditions
+    else:
+        conditions = set(['if', 'for', 'while', '&&', '||', '?', 'catch',
+                         'case'])
+    for token in tokens:
+        if token in conditions:
+            reader.context.add_condition()
+        yield token
+
+
+def recount_switch_case(tokens, reader):
+    for token in tokens:
+        if token == 'switch':
+            reader.context.add_condition()
+        elif token == 'case':
+            reader.context.add_condition(-1)
+        yield token
 
 
 class CodeReader(object):
@@ -352,16 +300,24 @@ class CodeReader(object):
     CodeReaders are used to parse functions structures from code of different
     language. Each language will need a subclass of CodeReader.
     '''
+    def __init__(self, context):
+        self.context = context
+        self._state = lambda _: _
+
     @staticmethod
     def compile_file_extension_re(*exts):
         return re.compile(r".*\.(" + r"|".join(exts) + r")$", re.IGNORECASE)
 
     @staticmethod
     def get_reader(filename):
+        # pylint: disable=E1101
         for lan in list(CodeReader.__subclasses__()):
             if CodeReader.compile_file_extension_re(*lan.ext).match(filename):
                 return lan
 
+    def state(self, token):
+        self._state(token)
+
     def eof(self):
         pass
 
@@ -388,7 +344,7 @@ class CodeReader(object):
         return token_pattern.findall(source_code)
 
 
-class CCppCommentsMixin(object):
+class CCppCommentsMixin(object):  # pylint: disable=R0903
     @staticmethod
     def get_comment_from_token(token):
         if token.startswith("/*") or token.startswith("//"):
@@ -396,207 +352,264 @@ class CCppCommentsMixin(object):
 
 
 try:
-    '''
-    lizard.py can run as a stand alone script, without the extensions
-    The following langauages / extensions will not be supported in
-    stand alone script.
-    '''
+    # lizard.py can run as a stand alone script, without the extensions
+    # The following langauages / extensions will not be supported in
+    # stand alone script.
     # pylint: disable=W0611
     from lizard_ext import JavaScriptReader
     from lizard_ext import PythonReader
-    from lizard_ext import CppNcssXMLFormatter
-except:
+    from lizard_ext import xml_output
+except ImportError:
     pass
 
 
 class CLikeReader(CodeReader, CCppCommentsMixin):
-    '''
-    This is the reader for C, C++ and Java.
-    '''
+    ''' This is the reader for C, C++ and Java. '''
 
     ext = ["c", "cpp", "cc", "mm", "cxx", "h", "hpp"]
     macro_pattern = re.compile(r"#\s*(\w+)\s*(.*)", re.M | re.S)
 
-    def __init__(self):
-        self.bracket_level = 0
+    def __init__(self, context):
+        super(CLikeReader, self).__init__(context)
+        self.bracket_stack = []
         self.br_count = 0
-        self.last_preprocessor = None
-        self._state = self._GLOBAL
+        self._state = self._state_global
+        self._saved_tokens = []
 
-    def preprocess(self, tokens, context):
+    def preprocess(self, tokens):
         for token in tokens:
             if not token.isspace() or token == '\n':
-                m = self.macro_pattern.match(token)
-                if m:
-                    yield "#" + m.group(1)
-                    param = m.group(2).strip()
-                    if param:
-                        yield param
+                macro = self.macro_pattern.match(token)
+                if macro:
+                    if macro.group(1) in ('if', 'ifdef', 'elif'):
+                        self.context.add_condition()
+                    for _ in macro.group(2).splitlines()[1:]:
+                        yield '\n'
                 else:
                     yield token
 
-    def _GLOBAL(self, token):
-        if token == '(':
-            self.bracket_level += 1
-            self._state = self._DEC
-            self.context.ADD_TO_LONG_FUNCTION_NAME(token)
-        elif token == '::':
-            self._state = self._NAMESPACE
-        else:
-            self.context.START_NEW_FUNCTION(token)
+    def _reset_to_global(self):
+        self._state = self._state_global
+        self.bracket_stack = []
+
+    def _state_global(self, token):
+        if token == 'typedef':
+            self._state = self._state_typedef
+        elif token[0].isalpha() or token[0] == '_':
+            self.context.start_new_function(token)
+            self._state = self._state_function
             if token == 'operator':
-                self._state = self._OPERATOR
+                self._state = self._state_operator
 
-    def _OPERATOR(self, token):
+    def _state_typedef(self, token):
+        if token == ';':
+            self._state = self._state_global
+
+    def _state_function(self, token):
+        if token == '(':
+            self.bracket_stack.append(token)
+            self._state = self._state_dec
+            self.context.add_to_long_function_name(token)
+        elif token == '::':
+            self._state = self._state_namespace
+        elif token == '<':
+            self._state = self._state_template_in_name
+            self.bracket_stack.append(token)
+            self.context.add_to_function_name(token)
+        else:
+            self._state = self._state_global
+            self._state_global(token)
+
+    def _state_template_in_name(self, token):
+        if token == "<":
+            self.bracket_stack.append(token)
+        elif token in (">", ">>"):
+            for _ in token:
+                if self.bracket_stack.pop() != "<":
+                    self._reset_to_global()
+        if not self.bracket_stack:
+            self._state = self._state_function
+        self.context.add_to_function_name(token)
+
+    def _state_operator(self, token):
         if token != '(':
-            self._state = self._GLOBAL
-        self.context.ADD_TO_FUNCTION_NAME(' ' + token)
+            self._state = self._state_operator_next
+        self.context.add_to_function_name(' ' + token)
 
-    def _NAMESPACE(self, token):
-        self._state = self._OPERATOR if token == 'operator' else self._GLOBAL
-        self.context.ADD_TO_FUNCTION_NAME("::" + token)
+    def _state_operator_next(self, token):
+        if token == '(':
+            self._state_function(token)
+        else:
+            self.context.add_to_function_name(' ' + token)
 
-    def _DEC(self, token):
+    def _state_namespace(self, token):
+        self._state = self._state_operator\
+            if token == 'operator' else self._state_function
+        self.context.add_to_function_name("::" + token)
+
+    def _state_dec(self, token):
         if token in ('(', "<"):
-            self.bracket_level += 1
-        elif token in (')', ">"):
-            self.bracket_level -= 1
-            if self.bracket_level == 0:
-                self._state = self._DEC_TO_IMP
-        elif self.bracket_level == 1:
-            self.context.PARAMETER(token)
+            self.bracket_stack.append(token)
+        elif token in (')', ">", ">>"):
+            for sub in token:
+                if self.bracket_stack.pop() != {')': '(', '>': '<'}[sub]:
+                    self._reset_to_global()
+                    return
+            if not self.bracket_stack:
+                self._state = self._state_dec_to_imp
+        elif len(self.bracket_stack) == 1:
+            self.context.parameter(token)
             return
-        self.context.ADD_TO_LONG_FUNCTION_NAME(" " + token)
+        self.context.add_to_long_function_name(" " + token)
 
-    def _DEC_TO_IMP(self, token):
+    def _state_dec_to_imp(self, token):
         if token == 'const':
-            self.context.ADD_TO_LONG_FUNCTION_NAME(" " + token)
+            self.context.add_to_long_function_name(" " + token)
+        elif token == 'throw':
+            self._state = self._state_throw
         elif token == '{':
             self.br_count += 1
-            self._state = self._IMP
+            self._state = self._state_imp
         elif token == ":":
-            self._state = self._CONSTRUCTOR_INITIALIZATION_LIST
+            self._state = self._state_initialization_list
+        elif not token[0].isalpha() or token[0] == '_':
+            self._state = self._state_global
         else:
-            self._state = self._GLOBAL
+            self._state = self._state_old_c_params
+            self._saved_tokens = [token]
 
-    def _CONSTRUCTOR_INITIALIZATION_LIST(self, token):
+    def _state_throw(self, token):
+        if token == ')':
+            self._state = self._state_dec_to_imp
+
+    def _state_old_c_params(self, token):
+        self._saved_tokens.append(token)
+        if token == ';':
+            self._state = self._state_dec_to_imp
+        elif token == '{':
+            if len(self._saved_tokens) == 2:
+                self._state_dec_to_imp(token)
+                return
+            self._state = self._state_global
+            for token in self._saved_tokens:
+                self._state(token)
+
+    def _state_initialization_list(self, token):
         if token == '{':
             self.br_count += 1
-            self._state = self._IMP
+            self._state = self._state_imp
 
-    def _IMP(self, token):
-        if token in ("#else", "#endif"):
-            self.last_preprocessor = token
-        # will ignore the braces in a #else branch
-        if self.last_preprocessor != '#else':
-            if token == '{':
-                self.br_count += 1
-            elif token == '}':
-                self.br_count -= 1
-                if self.br_count == 0:
-                    self._state = self._GLOBAL
-                    self.context.END_OF_FUNCTION()
+    def _state_imp(self, token):
+        if token == '{':
+            self.br_count += 1
+        elif token == '}':
+            self.br_count -= 1
+            if self.br_count == 0:
+                self._state = self._state_global
+                self.context.end_of_function()
 
 
 class JavaReader(CLikeReader, CodeReader):
 
     ext = ['java']
 
+    def _state_old_c_params(self, token):
+        if token == '{':
+            self._state_dec_to_imp(token)
+
 
 class ObjCReader(CLikeReader, CodeReader):
 
     ext = ['m']
 
-    def __init__(self):
-        super(ObjCReader, self).__init__()
+    def __init__(self, context):
+        super(ObjCReader, self).__init__(context)
 
-    def _DEC_TO_IMP(self, token):
+    def _state_global(self, token):
+        super(ObjCReader, self)._state_global(token)
+        if token == '(':
+            self.bracket_stack.append(token)
+            self._state = self._state_dec
+            self.context.add_to_long_function_name(token)
+
+    def _state_dec_to_imp(self, token):
         if token in ("+", "-"):
-            self._state = self._GLOBAL
+            self._state = self._state_global
         else:
-            super(ObjCReader, self)._DEC_TO_IMP(token)
-            if self._state == self._GLOBAL:
-                self._state = self._OBJC_DEC_BEGIN
-                self.context.START_NEW_FUNCTION(token)
+            super(ObjCReader, self)._state_dec_to_imp(token)
+            if self._state != self._state_imp:
+                self._state = self._state_objc_dec_begin
+                self.context.start_new_function(token)
 
-    def _OBJC_DEC_BEGIN(self, token):
+    def _state_objc_dec_begin(self, token):
         if token == ':':
-            self._state = self._OBJC_DEC
-            self.context.ADD_TO_FUNCTION_NAME(token)
+            self._state = self._state_objc_dec
+            self.context.add_to_function_name(token)
         elif token == '{':
             self.br_count += 1
-            self._state = self._IMP
+            self._state = self._state_imp
         else:
-            self._state = self._GLOBAL
+            self._state = self._state_global
 
-    def _OBJC_DEC(self, token):
+    def _state_objc_dec(self, token):
         if token == '(':
-            self._state = self._OBJC_PARAM_TYPE
-            self.context.ADD_TO_LONG_FUNCTION_NAME(token)
+            self._state = self._state_objc_param_type
+            self.context.add_to_long_function_name(token)
         elif token == ',':
             pass
         elif token == '{':
             self.br_count += 1
-            self._state = self._IMP
+            self._state = self._state_imp
         else:
-            self._state = self._OBJC_DEC_BEGIN
-            self.context.ADD_TO_FUNCTION_NAME(" " + token)
+            self._state = self._state_objc_dec_begin
+            self.context.add_to_function_name(" " + token)
 
-    def _OBJC_PARAM_TYPE(self, token):
+    def _state_objc_param_type(self, token):
         if token == ')':
-            self._state = self._OBJC_PARAM
-        self.context.ADD_TO_LONG_FUNCTION_NAME(" " + token)
+            self._state = self._state_objc_param
+        self.context.add_to_long_function_name(" " + token)
 
-    def _OBJC_PARAM(self, token):
-        self._state = self._OBJC_DEC
+    def _state_objc_param(self, _):
+        self._state = self._state_objc_dec
 
 
-class FunctionParser(object):
+def token_processor_for_function(tokens, reader):
+    ''' token_processor_for_function parse source code into functions. This is
+    different from language to language. So token_processor_for_function need
+    a language specific 'reader' to actually do the job.
     '''
-    FunctionParser parse source code into functions. This is different from
-    language to language. So FunctionParser need a language specific 'reader'
-    to actually do the job.
-    '''
-    def extend_tokens(self, tokens, context):
-        function_reader = context.reader
-        function_reader.context = context
-        for token in tokens:
-            function_reader._state(token)
-            yield token
-        function_reader.eof()
+    for token in tokens:
+        reader.state(token)
+        yield token
+    reader.eof()
 
 
-class FunctionOutputPlaceholder(object):
+class FileAnalyzer(object):  # pylint: disable=R0903
 
-    FUNCTION_CAPTION = " location  "
-    FUNCTION_INFO_PART = 'location'
-
-    def extend_tokens(self, tokens, context):
-        for token in tokens:
-            pass
-
-
-class FileAnalyzer(object):
-
-    def __init__(self, extensions=[]):
+    def __init__(self, extensions):
         self.processors = extensions
 
     def __call__(self, filename):
         try:
-            return self.analyze_source_code(filename, open(filename).read())
+            return self.analyze_source_code(
+                filename, open(filename, 'rU').read())
         except IOError:
-            sys.stderr.write("Error: Fail to read source file '%s'" % filename)
+            sys.stderr.write("Error: Fail to read source file '%s'\n"
+                             % filename)
 
     def analyze_source_code(self, filename, code):
         context = CodeInfoContext(filename)
-        tokens = context.reader.generate_tokens(code)
+        reader = (CodeReader.get_reader(filename) or CLikeReader)(context)
+        tokens = reader.generate_tokens(code)
         for processor in self.processors:
-            tokens = processor.extend_tokens(tokens, context)
+            tokens = processor(tokens, reader)
+        for _ in tokens:
+            pass
         return context.fileinfo
 
 
-def warning_filter(option, fileStatistics):
-    for file_info in fileStatistics:
+def warning_filter(option, module_infos):
+    for file_info in module_infos:
         if file_info:
             for fun in file_info.function_list:
                 if fun.cyclomatic_complexity > option.CCN or \
@@ -604,62 +617,71 @@ def warning_filter(option, fileStatistics):
                     yield fun
 
 
-class Whitelist(object):
-
-    def __init__(self, text):
-        self.whitelist = [
-            self._get_whitelist_item(line.split('#')[0])
-            for line in text.splitlines()]
-
-    def filter(self, warnings):
-        for warning in warnings:
-            if not self._in_list(warning):
-                yield warning
-
-    def _get_whitelist_item(self, text):
+def whitelist_filter(warnings, script=None):
+    def _get_whitelist_item(script):
         white = {}
-        pieces = text.replace('::', '##').split(':')
+        pieces = script.replace('::', '##').split(':')
         if len(pieces) > 1:
             white['file_name'] = pieces[0]
-            text = pieces[1]
+            script = pieces[1]
         white['function_names'] = (
-            [x.strip().replace('##', '::') for x in text.split(',')])
+            [x.strip().replace('##', '::') for x in script.split(',')])
         return white
 
-    def _in_list(self, warning):
-        return any(self._match_whitelist_item(white, warning)
-                   for white in self.whitelist)
+    def _in_list(warning):
+        return any(_match_whitelist_item(white, warning)
+                   for white in whitelist)
 
-    def _match_whitelist_item(self, white, warning):
+    def _match_whitelist_item(white, warning):
         return (warning.name in white['function_names'] and
                 warning.filename == white.get('file_name', warning.filename))
 
+    def get_whitelist():
+        whitelist_filename = "whitelizard.txt"
+        if os.path.isfile(whitelist_filename):
+            return open(whitelist_filename, mode='r').read()
+        return ''
 
-def print_function_info_header(extensions):
-    captions = "".join(
-        ext.FUNCTION_CAPTION for ext in extensions
-        if hasattr(ext, "FUNCTION_CAPTION"))
-    print("=" * len(captions))
-    print(captions)
-    print("-" * len(captions))
+    if not script:
+        script = get_whitelist()
+    whitelist = [
+        _get_whitelist_item(line.split('#')[0])
+        for line in script.splitlines()]
+    for warning in warnings:
+        if not _in_list(warning):
+            yield warning
 
 
-def get_extended_function_info(fun, extensions):
-    return ''.join(
-        str(getattr(fun, ext.FUNCTION_INFO_PART))
-        .rjust(len(ext.FUNCTION_CAPTION))
-        for ext in extensions if hasattr(ext, "FUNCTION_INFO_PART"))
+class OutputScheme(object):
+    def __init__(self, ext):
+        self.extensions = ext
+        self.items = [
+            {'caption': "  NLOC  ", 'value': "nloc"},
+            {'caption': "  CCN  ", 'value': "cyclomatic_complexity"},
+            {'caption': " token ", 'value': "token_count"},
+            {'caption': " PARAM ", 'value': "parameter_count"},
+            ] + [
+            {
+                'caption': ext.FUNCTION_CAPTION,
+                'value': ext.FUNCTION_INFO_PART
+            }
+            for ext in self.extensions if hasattr(ext, "FUNCTION_CAPTION")]
+        self.items.append({'caption': " location  ", 'value': 'location'})
+
+    def captions(self):
+        return "".join(item['caption'] for item in self.items)
+
+    def function_info_head(self):
+        captions = self.captions()
+        return "\n".join(("=" * len(captions), captions, "-" * len(captions)))
+
+    def function_info(self, fun):
+        return ''.join(
+            str(getattr(fun, item['value'])).rjust(len(item['caption']))
+            for item in self.items)
 
 
-def print_function_info(fun, extensions, warnings_only):
-    if warnings_only:
-        output_format = fun.clang_format_warning()
-    else:
-        output_format = get_extended_function_info(fun, extensions)
-    print(output_format)
-
-
-def print_warnings(option, warnings):
+def print_warnings(option, scheme, warnings):
     warning_count = 0
     if isinstance(option.sorting, list) and len(option.sorting) > 0:
         warnings = list(warnings)
@@ -669,14 +691,13 @@ def print_warnings(option, warnings):
         print(("\n" +
                "======================================\n" +
               "!!!! Warnings (CCN > %d) !!!!") % option.CCN)
-        print_function_info_header(option.extensions)
+        print(scheme.function_info_head())
     for warning in warnings:
         warning_count += 1
-        print_function_info(warning, option.extensions, option.warnings_only)
-
-    if warning_count == 0:
-        print("No warning found. Excellent!")
-
+        if option.warnings_only:
+            print(warning.clang_format_warning())
+        else:
+            print(scheme.function_info(warning))
     return warning_count
 
 
@@ -685,22 +706,21 @@ def print_total(warning_count, saved_result, option):
     all_fun = list(itertools.chain(*(file_info.function_list
                    for file_info in file_infos)))
     cnt = len(all_fun)
-    if (cnt == 0):
+    if cnt == 0:
         cnt = 1
-    files_NLOC = sum([f.nloc for f in file_infos])
-    functions_NLOC = sum([f.nloc for f in all_fun])
-    if (functions_NLOC == 0):
-        functions_NLOC = 1
+    nloc_in_functions = sum([f.nloc for f in all_fun])
+    if nloc_in_functions == 0:
+        nloc_in_functions = 1
     total_info = (
-        files_NLOC,
-        functions_NLOC / cnt,
+        sum([f.nloc for f in file_infos]),
+        nloc_in_functions / cnt,
         float(sum([f.cyclomatic_complexity for f in all_fun])) / cnt,
         float(sum([f.token_count for f in all_fun])) / cnt,
         cnt,
         warning_count,
         float(warning_count) / cnt,
         float(sum([f.nloc for f in all_fun
-              if f.cyclomatic_complexity > option.CCN])) / functions_NLOC
+              if f.cyclomatic_complexity > option.CCN])) / nloc_in_functions
         )
 
     if not option.warnings_only:
@@ -711,43 +731,45 @@ def print_total(warning_count, saved_result, option):
         print("%10d%10d%9.2f%11.2f%9d%13d%10.2f%8.2f" % total_info)
 
 
-def print_and_save_detail_information(allStatistics, extensions):
+def print_and_save_modules(all_modules, extensions, scheme):
     all_functions = []
-    print_function_info_header(extensions)
-    for fileStatistics in allStatistics:
+    print(scheme.function_info_head())
+    for module_info in all_modules:
         for extension in extensions:
             if hasattr(extension, 'reduce'):
-                extension.reduce(fileStatistics)
-        if fileStatistics:
-            all_functions.append(fileStatistics)
-            for fun in fileStatistics.function_list:
-                print(get_extended_function_info(fun, extensions))
+                extension.reduce(module_info)
+        if module_info:
+            all_functions.append(module_info)
+            for fun in module_info.function_list:
+                print(scheme.function_info(fun))
 
     print("--------------------------------------------------------------")
     print("%d file analyzed." % (len(all_functions)))
     print("==============================================================")
     print("NLOC    Avg.NLOC AvgCCN Avg.ttoken  function_cnt    file")
     print("--------------------------------------------------------------")
-    for fileStatistics in all_functions:
-        print(
-            "%7d%7d%7d%10d%10d     %s" % (
-                fileStatistics.nloc,
-                fileStatistics.average_NLOC,
-                fileStatistics.average_CCN,
-                fileStatistics.average_token,
-                len(fileStatistics.function_list),
-                fileStatistics.filename))
+    for module_info in all_functions:
+        print((
+            "{module.nloc:7d}" +
+            "{module.average_NLOC:7.0f}" +
+            "{module.average_CCN:7.1f}" +
+            "{module.average_token:10.0f}" +
+            "{function_count:10d}" +
+            "     {module.filename}").format(
+                module=module_info,
+                function_count=len(module_info.function_list)))
 
     return all_functions
 
 
 def print_result(code_infos, option):
+    scheme = OutputScheme(option.extensions)
     if not option.warnings_only:
-        code_infos = print_and_save_detail_information(
-            code_infos, option.extensions)
+        code_infos = print_and_save_modules(
+            code_infos, option.extensions, scheme)
     warnings = warning_filter(option, code_infos)
-    warnings = Whitelist(option.whitelist).filter(warnings)
-    warning_count = print_warnings(option, warnings)
+    warnings = whitelist_filter(warnings)
+    warning_count = print_warnings(option, scheme, warnings)
     print_total(warning_count, code_infos, option)
     for extension in option.extensions:
         if hasattr(extension, 'print_result'):
@@ -757,7 +779,7 @@ def print_result(code_infos, option):
 
 
 def print_xml(results, options):
-    print(CppNcssXMLFormatter().xml_output(list(results), options))
+    print(xml_output(list(results), options.verbose))
 
 
 def get_map_method(working_threads):
@@ -768,7 +790,7 @@ def get_map_method(working_threads):
         pool = multiprocessing.Pool(processes=working_threads)
         return pool.imap_unordered
     except ImportError:
-        return itertools.imap if sys.version[0] == '2' else map
+        return map
 
 
 def map_files_to_analyzer(files, file_analyzer, working_threads):
@@ -789,7 +811,7 @@ def md5_hash_file(full_path_name):
         return None
 
 
-def get_all_source_files(exclude_patterns, paths):
+def get_all_source_files(paths, exclude_patterns):
     '''
     Function counts md5 hash for the given file
     and checks if it isn't a duplicate using set
@@ -798,9 +820,9 @@ def get_all_source_files(exclude_patterns, paths):
     hash_set = set()
 
     def _validate_file(pathname):
-        return CodeReader.get_reader(pathname) and \
-            all(not fnmatch.fnmatch(pathname, p) for p in exclude_patterns) and \
-            _not_duplicate(pathname)
+        return (pathname in paths or (CodeReader.get_reader(pathname) and
+                all(not fnmatch(pathname, p) for p in exclude_patterns)
+                and _not_duplicate(pathname)))
 
     def _not_duplicate(full_path_name):
         fhash = md5_hash_file(full_path_name)
@@ -817,29 +839,17 @@ def get_all_source_files(exclude_patterns, paths):
                     for filename in files:
                         yield os.path.join(root, filename)
 
-    return (fn for fn in all_listed_files(paths)
-            if fn in paths or _validate_file(fn))
+    return filter(_validate_file, all_listed_files(paths))
 
 
 def parse_args(argv):
-
-    def get_whitelist():
-        whitelist_filename = "whitelizard.txt"
-        if os.path.isfile(whitelist_filename):
-            return open(whitelist_filename, mode='r').read()
-        return ''
-
-    options = createCommandLineParser(argv[0]).parse_args(args=argv[1:])
-    options.whitelist = get_whitelist()
-    options.extensions = get_extensions(options.extensions,
-                                        options.switchCasesAsOneCondition)
-    function_parts = [getattr(ext, 'FUNCTION_INFO_PART')
-                      for ext in options.extensions
-                      if hasattr(ext, 'FUNCTION_INFO_PART')]
+    options = create_command_line_parser(argv[0]).parse_args(args=argv[1:])
+    values = [
+        item['value'] for item in OutputScheme([]).items]
     for sort_factor in options.sorting:
-        if sort_factor not in function_parts:
+        if sort_factor not in values:
             error_message = "Wrong sorting field '%s'.\n" % sort_factor
-            error_message += "Candidates are: " + str(function_parts) + "\n"
+            error_message += "Candidates are: " + ', '.join(values) + "\n"
             sys.stderr.write(error_message)
             sys.exit(2)
     return options
@@ -848,27 +858,27 @@ def parse_args(argv):
 def get_extensions(extension_names, switch_case_as_one_condition=False):
     from importlib import import_module
     extensions = [
-        Preprocessor(),
-        CommentCounter(),
-        LineCounter(),
-        ConditionCounter(),
-        TokenCounter(),
-        FunctionParser(),
-        ParameterCounter()
+        preprocessing,
+        comment_counter,
+        line_counter,
+        condition_counter,
+        token_counter,
+        token_processor_for_function,
         ]
     if switch_case_as_one_condition:
-        extensions.append(SwitchCasesAsOneConditionCounter())
+        extensions.append(recount_switch_case)
 
     return extensions +\
-        [import_module('lizard_ext.lizard' + name).LizardExtension()
-            if isinstance(name, str) else name for name in extension_names] +\
-        [FunctionOutputPlaceholder()]
+        [import_module('lizard_ext.lizard' + name.lower()).LizardExtension()
+            if isinstance(name, str) else name for name in extension_names]
 
 analyze_file = FileAnalyzer(get_extensions([]))  # pylint: disable=C0103
 
 
 def lizard_main(argv):
     options = parse_args(argv)
+    options.extensions = get_extensions(options.extensions,
+                                        options.switchCasesAsOneCondition)
     printer = print_xml if options.xml else print_result
     result = analyze(
         options.paths,
